#pragma rtGlobals=1		// Use modern global access method.

Menu "Particle Analysis"
"ROI tools", PA_ROI_tools()
"Single image", PA_singleImage()
"Image series", PA_imageSeries()
end

Proc PA_ROI_tools()
	// recall data folder
	String dfStr=SIS_getDataFolderName()
	SetDataFolder root:$(dfStr):
	// recall ROI tools
	WMCreateImageROIPanel();
End

Function PA_singleImage()
	// set datafolder
	String dfStr=SIS_getDataFolderName() // where image is stored
	SetDataFolder root:$(dfStr)
	String imageStr=SIS_getImageStackName() // image name
	String winStr=SIS_getWinName() // nome della finestra
	//  input
	String objtypeStr="dark regions"
	variable objtype=1
	Prompt objtype,  "object type", popup "dark regions; bright regions;"
	variable mode=4
	Prompt mode, "threshold mode", popup "user level (0-255);halfway within ROI; V_avg of ROI; halfway within histogram;"
	variable thresh=128
	Prompt thresh, "threshold"
	variable minArea=10
	Prompt minArea, "minimum area"
	doPrompt  "input particle analysis", objtype, mode, thresh, minArea
	if (V_flag==0)
		if (exists("M_ROIMask")==1) // the ROI wave exists otherwise quit
			// bright or dark
			if (objtype ==2)
				objtypeStr="bright regions"
			endif
			// calc ROI area
			waveStats/Q M_ROIMask 
			variable areaTotROIMask=V_npnts-V_sum
			// calc threshold
			ImageTransform convert2gray $imageStr // creates binary image and store result in M_Image2grey
			if (mode > 1)
				ImageStats/R=M_ROIMask M_Image2Gray
				if (mode == 2)
					thresh = (V_max+V_min)/2
				endif
				if (mode == 3)
					thresh = V_rms
				endif
				if (mode == 4)
					ImageHistogram/R=M_ROIMask M_Image2Gray
					thresh = 0.5*(PA_histogramThreshold(W_ImageHist,0.03)+PA_histogramThreshold(W_ImageHist,0.97))
				endif
			endif
			if (objtype==1)
				ImageThreshold/M=0/T=(thresh)/Q/R=M_ROIMask M_Image2Gray
			else
				ImageThreshold/I/M=0/T=(thresh)/Q/R=M_ROIMask M_Image2Gray
			endif
			ImageAnalyzeParticles /W/Q/M=3/A=(minArea)/R=M_ROIMask/F stats, root:$(dfStr):M_ImageThresh
			PA_boundaryToFront(winStr,"W_BoundaryY","W_BoundaryX") //AppendToGraph/T root:$(dfStr):W_BoundaryY vs root:$(dfStr):W_BoundaryX
			// calculate and print total area particles within ROI
			variable areaParticles=sum(W_ImageObjArea)
			variable errorAreaParticles=sum(W_ImageObjPerimeter)
			String textBoxMessage="Object: "+objtypeStr+";\r"+"Threshold = "+num2Str(thresh)+"; Minimum Area = "+num2Str(minArea)+";\r"+"Total area = "+num2Str(areaParticles/areaTotROIMask*100)+"  "+num2Str(errorAreaParticles/areaTotROIMask*50)+" %"
			TextBox/C/N=areatext/A=LB "\\F'Arial'\\Z10"+textBoxMessage
			print ReplaceString("\r", textBoxMessage, " ")
		else
			print "please define ROI using ROI tools"
		endif
	endif
end

Function PA_imageSeries()
	// set datafolder
	String dfStr=SIS_getDataFolderName() // where image is stored
	SetDataFolder root:$(dfStr)
	String stackStr=SIS_getImageStackName() // image name
	String winStr=SIS_getWinName() // nome della finestra
	// global variables
	NVAR layer = root:$(dfStr):layer
	SVAR selectedwaveStr=root:$(stackStr):selectedwaveStr
	wave wv=$selectedwaveStr
	NVAR valdis = root:SIS:valdis
	//
	variable timeflag=0
	String unitsStr
	// 
	variable nlayers = dimSize($stackStr,2)
	// if nlayer == 1 call singleimage
	if (nlayers < 2)
		PA_singleImage()
	endif
	//  input
	String objtypeStr="dark regions"
	variable objtype=1
	Prompt objtype,  "object type", popup "dark regions; bright regions;"
	variable mode=5
	Prompt mode, "threshold mode", popup "user level (0-255);halfway within ROI; V_avg of ROI; halfway within histogram; relative to histogram maximum;"
	variable thresh=128
	Prompt thresh, "threshold"
	variable minArea=10
	Prompt minArea, "minimum area"
	variable from=layer
	Prompt from, "from layer"
	variable to=nlayers-1
	Prompt to, "to layer"
	variable makeMovie=1
	Prompt makeMovie,  "make movie", popup "no; yes;"
	variable fps=5
	Prompt fps, "movie speed (frame/s): "
	doPrompt  "input particle analysis", objtype, mode, thresh, minArea, from, to, makeMovie, fps
	if (V_flag==0)
		if (exists("M_ROIMask")==1) // the ROI wave exists otherwise quit
			if (makeMovie==2) // ask to save movie
				NewMovie /F=(fps)/L/O/Z/I as stackStr
				// define units
				if (stringmatch(selectedwaveStr,"pyro*")==1)
					unitsStr=" C"
				endif
				if (stringmatch(selectedwaveStr,"temp*")==1)
					unitsStr=" C"
				endif
				if (stringmatch(selectedwaveStr,"PMCH*")==1)
					unitsStr=" mbar"
				endif
				if (stringmatch(selectedwaveStr,"time*")==1)
					unitsStr=" s"
					timeflag=1
				endif
			endif
			// initialize waves
			String areaStr=dfStr+"_AR"
			String perimeterStr=dfStr+"_PE"
			String areaerrorStr=dfStr+"_ARE"
			make/n=(nlayers)/O  $areaStr=NaN,   $areaerrorStr=NaN,  $perimeterStr=NaN
			wave aw=$areaStr
			wave aew=$areaerrorStr
			wave pw=$perimeterStr
			// initialize other variables
			if (objtype ==2)
				objtypeStr="bright regions"
			endif
			// calc ROI area
			waveStats/Q M_ROIMask 
			variable areaTotROIMask=V_npnts-V_sum
			// calc threshold for the selected layer
			ImageTransform/P=(layer)/PTYP=0 getPlane $stackStr // puts layer into M_ImagePlane
			ImageTransform convert2gray M_ImagePlane // creates binary image and store result in M_Image2grey
			variable hmax=NaN
			if (mode > 1)
				ImageStats/R=M_ROIMask M_Image2Gray
				if (mode == 2)
					thresh = (V_max+V_min)/2
				endif
				if (mode == 3)
					thresh = V_rms
				endif
				if ((mode == 4) || (mode == 5))
					ImageHistogram/R=M_ROIMask M_Image2Gray
					thresh = 0.5*(PA_histogramThreshold(W_ImageHist,0.03)+PA_histogramThreshold(W_ImageHist,0.97))
					wavestats/Q W_ImageHist
					hmax=V_maxloc
				endif
			endif
			// loop
			variable i=from
			variable thresh_i
			String textBoxMessage=""
			variable unitstep= 1 
			if (to<from)
				unitstep=-1
			endif
			do
				// sho layer
				// layer=i
				modifyImage $(stackStr) plane=(i)
				valdis=wv[i]
				ValDisplay vd,value=#num2Str(valdis)
				// layer operations
				ImageTransform/P=(i)/PTYP=0 getPlane $stackStr // get layer
				ImageTransform convert2gray M_ImagePlane
				// threshold
				if (mode == 5)
					ImageHistogram/R=M_ROIMask M_Image2Gray
					WaveStats/Q W_ImageHist
					thresh_i = thresh*V_maxloc/hmax
				else
					thresh_i=thresh
				endif
				// calculate threshold image for bright or dark particles
				if (objtype==1)
					ImageThreshold/M=0/T=(thresh_i)/Q/R=M_ROIMask M_Image2Gray
				else
					ImageThreshold/I/M=0/T=(thresh_i)/Q/R=M_ROIMask M_Image2Gray
				endif 
				ImageAnalyzeParticles /W/Q/M=3/A=(minArea)/R=M_ROIMask/F stats, root:$(dfStr):M_ImageThresh
				// update if i==from
				if (i==from)
					PA_boundaryToFront(winStr,"W_BoundaryY","W_BoundaryX") 
				endif
				// calculate and print total area particles within ROI
				aw[i]=sum(W_ImageObjArea)/areaTotROIMask*100
				aew[i]=sum(W_ImageObjPerimeter)/areaTotROIMask*50
				pw[i]=sum(W_ImageObjPerimeter)/areaTotROIMask*100
				// print texbox on stack 
				if (makeMovie==1)
					textBoxMessage="i = "+num2Str(i)+"; Object: "+objtypeStr+";\r"+"Threshold = "+num2Str(thresh)+"; Minimum Area = "+num2Str(minArea)+";\r"+"Total area = "+num2Str(aw[i])+"  "+num2Str(pw[i])+" %"
					TextBox/C/N=areatext/A=LB "\\F'Arial'\\Z10"+textBoxMessage
					print ReplaceString("\r", textBoxMessage, " ")
				else
					if (timeflag==1)
						sprintf textbox2Str "%*.*f" 4, 2, wv[i]/1000
					else
						sprintf textboxMessage "%*.*f" 4, 2, wv[i]
					endif
					TextBox/C/N=areatext/A=LB textboxMessage+unitsStr
				endif
				// update
				doUpdate
				// movie
				if (makeMovie==2) //  save movie frame
					AddMovieFrame
				endif
				// loop
				i+=unitstep
			while ((i<=abs(from-to)) && (i>-1))
			if (makeMovie==2) // close movie
				closeMovie
			endif
			// restore layer
			modifyImage $(stackStr) plane=(from)
			valdis=wv[from]
			ValDisplay vd,value=#num2Str(valdis)
			removeFromGraph W_BoundaryY
			TextBox/K/N=areatext
			doUpdate
			// display results if ...
			variable axis=3
			Prompt axis, "x axis", popup "image number;time;temperature (TC);temperature (PY);other;"
			doPrompt "axes label" axis
			if (V_flag==0)	
				String xStr
				String graphStr="PA"+dfStr
				String paxStr="PA"+dfStr+"_P"
				// image number
				if (axis == 1)
					Prompt xStr, "enter wave",popup WaveList("*NUM*",";","")
					doPrompt "bottom axis input wave" xStr
					if (V_flag==0)	
						duplicate/o $xStr $paxStr
						SIS_xywaveToLeftAxis(graphStr,areaStr,paxStr)
						Label bottom "\\F'Arial'\\Z16Image number"
					endif
				endif
				// time
				if (axis == 2)
					Prompt xStr, "enter wave",popup WaveList("ti*",";","")
					doPrompt "bottom axis input wave" xStr
					if (V_flag==0)	
						duplicate/o $xStr $paxStr
						wave paxw=$paxStr
						paxw*=1000 
						SIS_xywaveToLeftAxis(graphStr,areaStr,paxStr)
						Label bottom "\\F'Arial'\\Z16Time (s)"
					endif
				endif
				// temperature (TC)
				if (axis == 3)
					Prompt xStr, "enter wave",popup WaveList("te*",";","")
					doPrompt "bottom axis input wave" xStr
					if (V_flag==0)	
						duplicate/o $xStr $paxStr
						SIS_xywaveToLeftAxis(graphStr,areaStr,paxStr)
						Label bottom "\\F'Arial'\\Z16Temperature (C)"
					endif
				endif
				// temperature (Pyrometer)
				if (axis == 4)
					Prompt xStr, "enter wave",popup WaveList("py*",";","")
					doPrompt "bottom axis input wave" xStr
					if (V_flag==0)	
						duplicate/o $xStr $paxStr
						SIS_xywaveToLeftAxis(graphStr,areaStr,paxStr)
						Label bottom "\\F'Arial'\\Z16Temperature (C)"
					endif
				endif				
				// other
				if (axis == 5)
					Prompt xStr, "enter wave",popup WaveList("*",";","")
					doPrompt "bottom axis input wave" xStr
					if (V_flag==0)	
						duplicate/o $xStr $paxStr
						SIS_xywaveToLeftAxis(graphStr,areaStr,paxStr)
						Label bottom "\\F'Arial'\\Z16Other"
					endif
				endif
				ModifyGraph fSize=12
				Label left "\\F'Arial'\\Z16Area fraction (%)"
				ModifyGraph mode=4,marker=8;DelayUpdate
				ErrorBars $areaStr Y,wave=($areaerrorStr,$areaerrorStr)
			endif
		else
			print "please define ROI using ROI tools"
		endif
	endif
end

Function PA_histogramThreshold(hw,thresh)
	wave hw
	variable thresh
	variable s=0,i=0
	variable tot= sum(hw)
	do
		s += hw[i]
		i+=1
	while( (s/tot) < thresh )
	return i
end

Function PA_boundaryToFront(graphNameStr,waveNameStr,wavexNameStr)
	String graphNameStr
	String waveNameStr
	String wavexNameStr
	graphNameStr= SIS_removeQuotes(graphNameStr)
	Dowindow $graphNameStr
	if (V_flag==0)
		Display
		DoWindow/C $graphNameStr
		appendToGraph/T $waveNameStr vs $wavexNameStr
		ModifyGraph width=240,height={Aspect,1}
		ModifyGraph mirror=1
	else
		DoWindow/F $graphNameStr
		String empty=WaveList(waveNameStr, ";", "WIN:"+graphNameStr)
		if (strlen(empty)==0)
			appendToGraph/T $waveNameStr vs $wavexNameStr
		endif
	endif
end

End

